<?php

class QUESTIONS_BOL_FollowDao extends OW_BaseDao
{
    /**
     * Singleton instance.
     *
     * @var QUESTIONS_BOL_FollowDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return QUESTIONS_BOL_FollowDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'QUESTIONS_BOL_Follow';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'questions_follow';
    }

    public function addFollow( $userId, $questionId )
    {
        $dto = $this->findFollow($userId, $questionId);

        if ( $dto === null )
        {
            $dto = new QUESTIONS_BOL_Follow();
            $dto->userId = $userId;
            $dto->timeStamp = time();
            $dto->questionId = $questionId;
            $this->save($dto);
        }

        return $dto;
    }

    public function findFollow( $userId, $questionId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('userId', $userId);
        $example->andFieldEqual('questionId', $questionId);

        return $this->findObjectByExample($example);
    }

    public function findByQuestionId( $questionId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('questionId', $questionId);

        return $this->findListByExample($example);
    }

    public function findFollowCount( $questionId, $userContext = array(), $ignoreUsers = array() )
    {
        $notIn = '1';

        if ( !empty($ignoreUsers) )
        {
            $notIn = 'userId NOT IN ("' . implode('","', $ignoreUsers) . '")';
        }

        $query = 'SELECT COUNT(*) FROM ' . $this->getTableName() . ' WHERE questionId=:q AND ' . $notIn;

        $out = $this->dbo->queryForColumn($query, array(
            'q' => $questionId
        ));

        return $out;
    }

    public function findFollowList( $questionId, $userContext = array(), $ignoreUsers = array() )
    {
        $notIn = '1';
        $order = 'timeStamp DESC';

        if ( !empty($ignoreUsers) )
        {
            $notIn = 'userId NOT IN ("' . implode('","', $ignoreUsers) . '")';
        }

        if ( !empty($userContext) )
        {
            $order = 'IF( userId IN ("' . implode('","', $userContext) . '"), 1, 0) DESC';
        }

        $query = 'SELECT * FROM ' . $this->getTableName() . ' WHERE questionId=:q AND ' . $notIn . ' ORDER BY ' . $order;

        $out = $this->dbo->queryForObjectList($query, $this->getDtoClassName(), array(
            'q' => $questionId
        ));

        return $out;
    }

    public function removeFollow( $userId, $questionId )
    {
        $example = new OW_Example();
        $example->andFieldEqual('userId', $userId);
        $example->andFieldEqual('questionId', $questionId);

        return $this->deleteByExample($example);
    }
    
    public function findByUserId( $userId )
    {
        $example = new OW_Example();
        $example->andFieldEqual("userId", $userId);
        
        return $this->findListByExample($example);
    }
}