<?php
class QUESTIONS_CLASS_ActivityBridge
{
    /**
     * Singleton instance.
     *
     * @var QUESTIONS_CLASS_ActivityBridge
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return QUESTIONS_CLASS_ActivityBridge
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     *
     * @var QUESTIONS_BOL_FeedService
     */
    private $service;

    private function __construct()
    {
        $this->service = QUESTIONS_BOL_FeedService::getInstance();
    }

    public function onQuestionAdd( OW_Event $e )
    {
        $params = $e->getParams();

        $activity = new QUESTIONS_BOL_Activity();
        $activity->questionId = (int) $params['id'];
        $activity->activityType = QUESTIONS_BOL_FeedService::ACTIVITY_CREATE;
        $activity->activityId = (int) $params['id'];
        $activity->userId = (int) $params['userId'];
        $activity->privacy = $params['privacy'];
        $activity->timeStamp = time();

        $this->service->saveActivity($activity);
    }

    public function onQuestionRemove( OW_Event $e )
    {
        $params = $e->getParams();
        $this->service->deleteActivity($params['id'], QUESTIONS_BOL_FeedService::ACTIVITY_CREATE, $params['id']);
    }

    public function onAnswerAdd( OW_Event $e )
    {
        $params = $e->getParams();
        $option = QUESTIONS_BOL_Service::getInstance()->findOption($params['optionId']);

        if ( $option === null )
        {
            return;
        }

        $activity = new QUESTIONS_BOL_Activity();
        $activity->questionId = $option->questionId;
        $activity->activityType = QUESTIONS_BOL_FeedService::ACTIVITY_ANSWER;
        $activity->activityId = (int) $params['id'];
        $activity->userId = (int) $params['userId'];
        $activity->timeStamp = time();
        $activity->setData(array(
            'text' => $option->text,
            'optionId' => $option->id
        ));

        $this->service->saveActivity($activity);
    }

    public function onAnswerRemove( OW_Event $e )
    {
        $params = $e->getParams();
        $option = QUESTIONS_BOL_Service::getInstance()->findOption($params['optionId']);

        if ( $option === null )
        {
            return;
        }

        $this->service->deleteActivity($option->questionId, QUESTIONS_BOL_FeedService::ACTIVITY_ANSWER, $params['id']);
    }

    public function onFollowAdd( OW_Event $e )
    {
        $params = $e->getParams();

        $activity = new QUESTIONS_BOL_Activity();
        $activity->questionId = (int) $params['questionId'];
        $activity->activityType = QUESTIONS_BOL_FeedService::ACTIVITY_FOLLOW;
        $activity->activityId = (int) $params['userId'];
        $activity->userId = (int) $params['userId'];
        $activity->timeStamp = time();

        $this->service->saveActivity($activity);
    }

    public function onFollowRemove( OW_Event $e )
    {
        $params = $e->getParams();
        $this->service->deleteActivity($params['questionId'], QUESTIONS_BOL_FeedService::ACTIVITY_FOLLOW, $params['userId']);
    }

    public function onPostAdd( OW_Event $e )
    {
        $params = $e->getParams();

        $activity = new QUESTIONS_BOL_Activity();
        $activity->questionId = (int) $params['questionId'];
        $activity->activityType = QUESTIONS_BOL_FeedService::ACTIVITY_POST;
        $activity->activityId = (int) $params['id'];
        $activity->userId = (int) $params['userId'];
        $activity->timeStamp = time();

        $activity->setData(array(
            'text' => $params['text']
        ));

        $this->service->saveActivity($activity);
    }

    public function onPostRemove( OW_Event $e )
    {
        $params = $e->getParams();
        $this->service->deleteActivity($params['questionId'], QUESTIONS_BOL_FeedService::ACTIVITY_POST, $params['id']);
    }
}