<?php
class QUESTIONSANSW_CLASS_CommentsBridge
{
    /**
     * Singleton instance.
     *
     * @var QUESTIONSANSW_CLASS_CommentsBridge
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return QUESTIONSANSW_CLASS_CommentsBridge
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     *
     * @var BOL_CommentService
     */
    private $service;

    private function __construct()
    {
        $this->service = BOL_CommentService::getInstance();
    }

    public function onCommentAdd( OW_Event $e )
    {
        $params = $e->getParams();

        if ( $params['entityType'] != QUESTIONSANSW_BOL_Service::ENTITY_TYPE )
        {
            return;
        }

        $questionId = (int) $params['entityId'];
        $comment = $this->service->findComment($params['commentId']);

        $event = new OW_Event(QUESTIONSANSW_BOL_Service::EVENT_POST_ADDED, array(
            'questionId' => $questionId,
            'id' => (int) $params['commentId'],
            'userId' => (int) $params['userId'],
            'text' => $comment->message
        ));

        OW::getEventManager()->trigger($event);
    }

    public function onCommentRemove( OW_Event $e )
    {
        $params = $e->getParams();

        if ( $params['entityType'] != QUESTIONSANSW_BOL_Service::ENTITY_TYPE )
        {
            return;
        }

        $questionId = (int) $params['entityId'];

        $event = new OW_Event(QUESTIONSANSW_BOL_Service::EVENT_POST_REMOVED, array(
            'questionId' => $questionId,
            'id' => (int) $params['commentId'],
            'userId' => (int) $params['userId']
        ));

        OW::getEventManager()->trigger($event);
    }

    public function onQuestionRemove( OW_Event $e )
    {
        $params = $e->getParams();
        $questionId = (int) $params['id'];

        $posts = $this->service->findFullCommentList(QUESTIONSANSW_BOL_Service::ENTITY_TYPE, $questionId);

        foreach ( $posts as $post )
        {
            $event = new OW_Event(QUESTIONSANSW_BOL_Service::EVENT_POST_REMOVED, array(
                'questionId' => $questionId,
                'id' => $post->id,
                'userId' => $post->userId
            ));

            OW::getEventManager()->trigger($event);
        }

        $this->service->deleteEntityComments(QUESTIONSANSW_BOL_Service::ENTITY_TYPE, $questionId);
    }
}