<?php
class QUESTIONSANSW_CLASS_NewsfeedBridge
{
    /**
     * Singleton instance.
     *
     * @var QUESTIONSANSW_CLASS_NewsfeedBridge
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return QUESTIONSANSW_CLASS_NewsfeedBridge
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     *
     * @var QUESTIONSANSW_BOL_Service
     */
    private $service;

    private function __construct()
    {
        $this->service = QUESTIONSANSW_BOL_Service::getInstance();
    }

    //Event Handlers

    public function onStatusCmp( OW_Event $event )
    {
        $params = $event->getParams();
        $language = OW::getLanguage();

        $status = $event->getData();
        
        if ( $status === null )
        {
            $status = OW::getClassInstance("NEWSFEED_CMP_UpdateStatus", $params['feedAutoId'], $params['entityType'], $params['entityId'], $params['visibility']);
        }

        if ( QUESTIONSANSW_BOL_Service::getInstance()->isCurrentUserCanAsk() )
        {
            $tabs = new QUESTIONSANSW_CMP_Tabs();
            $tabs->addTab($language->text('questionsansw', 'newsfeed_status_tab'), $status, 'ow_ic_chat');
            $question = new QUESTIONSANSW_CMP_NewsfeedQuestionAdd($params['feedAutoId'], $params['entityType'], $params['entityId'], $params['visibility']);
            $tabs->addTab($language->text('questionsansw', 'newsfeed_question_tab'), $question, 'ow_ic_lens');

            $status = $tabs;
        }

        $event->setData($status);
        
        return $status;
    }

    public function onItemRender( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();

        if ( $params['action']['entityType'] != QUESTIONSANSW_BOL_Service::ENTITY_TYPE )
        {
            return;
        }

        $language = OW::getLanguage();
        $configs = OW::getConfig()->getValues('questionsansw');
        $questionId = $params['action']['entityId'];
        $userId = OW::getUser()->getId();

        $question = $this->service->findQuestion($questionId);
        $optionTotal = $this->service->findOptionCount($questionId);
        $answerCount = $this->service->findTotalAnswersCount($questionId);
        $postCount = BOL_CommentService::getInstance()->findCommentCount(QUESTIONSANSW_BOL_Service::ENTITY_TYPE, $params['action']['entityId']);
        $userContext = array();

        $count = QUESTIONSANSW_BOL_Service::DISPLAY_COUNT;
        if ( $optionTotal - $count < 2 )
        {
            $count = $optionTotal;
        }

        $cmp = new QUESTIONSANSW_CMP_Answers($question, $optionTotal, array(0, $count));
        $cmp->setTotalAnswerCount($answerCount);

        if ( in_array($params['feedType'], array('user', 'my')) )
        {
            foreach ( $params['activity'] as $act )
            {
                if ( $act['activityType'] == 'answer' )
                {
                    $userContext[] = $act['userId'];
                }
            }

            $cmp->setUsersContext($userContext);
        }

        $lastActivity = $this->getBubbleActivity($params);

        $data['assign']['answers'] = $cmp->render();

        $questionUrl = OW::getRouter()->urlForRoute('questionsansw-question', array(
            'qid' => $question->id
        ));

        $jsSelector = 'QUESTIONSANSW_AnswerListCollection.' . $cmp->getUniqId();
        $allowPopups = !isset($configs['allow_popups']) || $configs['allow_popups'];

        $data['assign']['string'] = $data['string'] = $this->getItemString($question, $lastActivity, $jsSelector, $questionUrl);

        $data['features'] = array();

        $onClickStr = "window.location.href='$questionUrl'";

        if ( $configs['allow_comments'] )
        {
            if ( $allowPopups )
            {
                $onClickStr = "return {$jsSelector}.openQuestionDelegate(true);";
            }

            $data['features']["q-comments"] = array(
                'class' => 'q-' . $cmp->getUniqId() . '-status-comments',
                'iconClass' => 'ow_miniic_comment',
                'label' => $postCount,
                'onclick' => $onClickStr,
                'string' => null
            );
        }

        if ( $allowPopups )
        {
            $onClickStr = "return {$jsSelector}.openQuestionDelegate();";
        }

        $data['features']["q-votes"] = array(
            'class' => 'q-' . $cmp->getUniqId() . '-status-votes',
            'iconClass' => 'questionsansw_miniicon_check',
            'label' => $answerCount,
            'onclick' => $onClickStr,
            'string' => null
        );

        if ( $configs['enable_follow'] )
        {
            $onClickStr = "OW.error('" . $language->text('questionsansw', 'follow_not_allowed') . "')";
            $isFollowing = false;

            if ( $this->service->isCurrentUserCanInteract($question) )
            {
                $isFollowing = $this->service->isFollow($userId, $question->id);
                $onClickStr = $isFollowing
                    ? $jsSelector . '.unfollowQuestion();'
                    : $jsSelector . '.followQuestion();';
            }
            else if ( OW::getUser()->isAuthenticated() )
            {
                $isFollowing = $this->service->isFollow($userId, $question->id);

                if ( $isFollowing )
                {
                    $onClickStr = $jsSelector . '.unfollowQuestion();';
                }
            }

            $data['features']["q-follows"] = array(
                'class' => 'q-' . $cmp->getUniqId() . '-status-follows',
                'iconClass' => 'questionsansw_miniic_follow',
                'label' => $this->service->findFollowsCount($question->id),
                'onclick' => $onClickStr,
                'active' => $isFollowing
            );
        }
        
        $settings = $question->getSettings();

        $selfContext = isset($settings['context']['type']) && $settings['context']['type'] == "user" && $settings['context']['id'] == $question->userId;
        
        if ( !$selfContext && isset($settings['context']) && $settings['context']['type'] != $params['feedType'] )
        {
            if ( !empty($settings['context']['url']) && !empty($settings['context']['label']) )
            {
                $data['context'] = $settings['context'];
            }
        }
        
        if ( $selfContext ) 
        {
            $data['context'] = null;
        }
        
        $event->setData($data);
    }

    private function getBubbleActivity( $params )
    {
        if ( !empty($params['lastActivity']) )
        {
            return $params['lastActivity'];
        }

        foreach ( $params['activity'] as $act ) //TODO: Back compatibility with 1.3.1
        {
            if ( !in_array($act['activityType'], array('subscribe')) )
            {
                return $act;
            }
        }

        return $params['createActivity'];
    }

    private function getItemString( $question, $bubbleActivity, $jsSelector, $questionUrl )
    {
        $activityType = $bubbleActivity['activityType'];

        $configs = OW::getConfig()->getValues('questionsansw');
        $allowPopups = !isset($configs['allow_popups']) || $configs['allow_popups'];
        $onClickStr = $allowPopups ? 'onclick="return ' . $jsSelector . '.openQuestionDelegate();"' : '';

        $questionEmbed = '<a href="' . $questionUrl . '" ' . $onClickStr . '>' . $question->text . '</a>';

        if ( in_array($activityType, array(QUESTIONSANSW_BOL_FeedService::ACTIVITY_CREATE, QUESTIONSANSW_BOL_FeedService::ACTIVITY_FOLLOW)) )
        {
            return OW::getLanguage()->text('questionsansw', 'item_text_' . $activityType, array(
                'question' => $questionEmbed
            ));
        }

        $buubleData = $bubbleActivity['data'];
        $with = '';
        if ( !empty($buubleData['text']) )
        {
            $text = UTIL_String::truncate($buubleData['text'], 50, '...');
            $with = '<a href="' . $questionUrl . '" ' . $onClickStr . '>' . $text . '</a>';
        }

        return OW::getLanguage()->text('questionsansw', 'item_text_' . $activityType, array(
            'question' => $questionEmbed,
            'with' => $with
        ));
    }

    public function onEntityAdd( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();
        $language = OW::getLanguage();

        if ( $params['entityType'] != QUESTIONSANSW_BOL_Service::ENTITY_TYPE )
        {
            return;
        }

        $questionId = (int) $params['entityId'];
        $question = QUESTIONSANSW_BOL_Service::getInstance()->findQuestion($questionId);

        if ( $question === null )
        {
            return;
        }

        $questionUrl = OW::getRouter()->urlForRoute('questionsansw-question', array(
            'qid' => $question->id
        ));

        $questionEmbed = '<a href="' . $questionUrl . '">' . $question->text . '</a>';
        $string = $language->text('questionsansw', 'item_text_create', array(
            'question' => $questionEmbed
        ));

        $data = array_merge($data, array(
            'params' => array(
                'subscribe' => true
            ),
            'ownerId' => (int) $question->userId,
            'time' => (int) $question->timeStamp,
            'string' => $string,
            'content' => '[ph:answers]',
            'view' => array(
                'iconClass' => 'ow_ic_lens'
            ),
            'data' => array(
                'questionId' => $question->id
            )
        ));

        $event->setData($data);
    }

    public function onActivity( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();

        $siteActivity = true;

        if ( !$siteActivity )
        {
            if ( $params['entityType'] == QUESTIONSANSW_BOL_Service::ENTITY_TYPE )
            {
                if ( $params['activityType'] == 'comment' )
                {
                    $data['params']['subscribe'] = false;
                }

                if ( !isset($params['visibility']) && !isset($data['params']['visibility']) )
                {
                    $data['params']['visibility'] = 15;
                }
                else if (isset($params['visibility']))
                {
                    $data['params']['visibility'] = $params['visibility'];
                }

                if ( $params['activityType'] != 'create' && intval($data['params']['visibility']) & 1 )
                {
                    $data['params']['visibility'] -= 1; // All visibility (15) instead of SITE Visibility (1)
                }
            }
        }

        $event->setData($data);
    }

    public function onAnswerAdd( OW_Event $event )
    {
        $params = $event->getParams();
        $optionId = (int) $params['optionId'];

        $optionDto = QUESTIONSANSW_BOL_Service::getInstance()->findOption($optionId);

        $id = (int) $params['id'];

        $activityParams = array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $optionDto->questionId,
            'pluginKey' => 'questionsansw',
            'activityType' => QUESTIONSANSW_BOL_FeedService::ACTIVITY_ANSWER,
            'activityId' => $id,
            'userId' => $params['userId']
        );

        $activityData = array(
            'answerId' => $id,
            'optionId' => $optionId,
            'text' => $optionDto->text,
            'string' => '[ph:string]'
        );

        $event = new OW_Event('feed.activity', $activityParams, $activityData);
        OW::getEventManager()->trigger($event);
    }

    public function onAnswerRemove(  OW_Event $event )
    {
        $params = $event->getParams();

        $optionId = (int) $params['optionId'];
        $optionDto = QUESTIONSANSW_BOL_Service::getInstance()->findOption($optionId);

        $activityParams = array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $optionDto->questionId,
            'activityType' => QUESTIONSANSW_BOL_FeedService::ACTIVITY_ANSWER,
            'activityId' => $params['id']
        );

        $event =new OW_Event('feed.delete_activity', $activityParams);
        OW::getEventManager()->trigger($event);
    }

    public function onFollowAdd( OW_Event $event )
    {
        $params = $event->getParams();

        $activityParams = array(
            'pluginKey' => 'questionsansw',
            'userId' => $params['userId'],
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $params['questionId'],
            'activityId' => $params['userId'],
            'activityType' => 'subscribe',
            'visibility' => 14, // Visibility autor
            'time' => time()
        );

        $event =new OW_Event('feed.activity', $activityParams);
        OW::getEventManager()->trigger($event);

        $activityParams['activityType'] = QUESTIONSANSW_BOL_FeedService::ACTIVITY_FOLLOW;
        $activityParams['activityId'] = $params['id'];
        $activityParams['subscribe'] = false;

        $activityData = array(
            'string' => '[ph:string]'
        );

        $event = new OW_Event('feed.activity', $activityParams, $activityData);
        OW::getEventManager()->trigger($event);
    }

    public function onFollowRemove( OW_Event $event )
    {
        $params = $event->getParams();

        $activityParams = array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $params['questionId'],
            'activityType' => 'subscribe',
            'activityId' => $params['userId']
        );

        $event =new OW_Event('feed.delete_activity', $activityParams);
        OW::getEventManager()->trigger($event);

        $activityParams['activityType'] = QUESTIONSANSW_BOL_FeedService::ACTIVITY_FOLLOW;
        $activityParams['activityId'] = $params['id'];
        $activityParams['subscribe'] = false;

        $event =new OW_Event('feed.delete_activity', $activityParams);
        OW::getEventManager()->trigger($event);
    }

    public function onPostAdd( OW_Event $e )
        {
        $params = $e->getParams();

        $activityParams = array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => (int) $params['questionId'],
            'pluginKey' => 'questionsansw',
            'activityType' => QUESTIONSANSW_BOL_FeedService::ACTIVITY_POST,
            'activityId' => (int) $params['id'],
            'userId' => $params['userId']
        );

        $activityData = array(
            'text' => $params['text'],
            'string' => '[ph:string]'
        );

        $event = new OW_Event('feed.activity', $activityParams, $activityData);
        OW::getEventManager()->trigger($event);
        }

    public function onPostRemove( OW_Event $e )
    {
        $params = $e->getParams();

        $activityParams = array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $params['questionId'],
            'activityType' => QUESTIONSANSW_BOL_FeedService::ACTIVITY_POST,
            'activityId' => $params['id']
        );

        $event =new OW_Event('feed.delete_activity', $activityParams);
        OW::getEventManager()->trigger($event);
    }

    public function onQuestionRemove( OW_Event $event )
    {
        $params = $event->getParams();

        $questionId = $params['id'];

        $event = new OW_Event('feed.delete_item', array(
            'entityType' => QUESTIONSANSW_BOL_Service::ENTITY_TYPE,
            'entityId' => $questionId
        ));

        OW::getEventManager()->trigger($event);
    }

    public function configurableActivity( BASE_CLASS_EventCollector $event )
    {
        $language = OW::getLanguage();
        $event->add(array(
            'label' => $language->text('questionsansw', 'feed_content_label'),
            'activity' => '*:' . QUESTIONSANSW_BOL_Service::ENTITY_TYPE
        ));
    }

    public function collectPrivacy( BASE_CLASS_EventCollector $event )
    {
        $event->add(array('*:' . QUESTIONSANSW_BOL_Service::ENTITY_TYPE, QUESTIONSANSW_Plugin::PRIVACY_ACTION_VIEW_MY_QUESTIONSANSW));
    }
}