<?php
class QUESTIONSANSW_CMP_QuestionAdd extends OW_Component
{
    public function __construct()
    {
        parent::__construct();

        if ( !QUESTIONSANSW_BOL_Service::getInstance()->isCurrentUserCanAsk() )
        {
            $this->setVisible(false);

            return;
        }

        $template = OW::getPluginManager()->getPlugin('questionsansw')->getCmpViewDir() . 'question_add.html';
        $this->setTemplate($template);
    }

    public function onBeforeRender()
    {
        parent::onBeforeRender();

        $uniqId = uniqid('questionAdd');
        $this->assign('uniqId', $uniqId);

        $config = OW::getConfig()->getValues(QUESTIONSANSW_Plugin::PLUGIN_KEY);

        $this->assign('configs', $config);

        $form = $this->initForm();
        $this->addForm($form);

        QUESTIONSANSW_Plugin::getInstance()->addStatic();

        $js = UTIL_JsGenerator::newInstance()->newObject('questionsanswAdd', 'QUESTIONS_QuestionAdd', array($uniqId, $form->getName(), array(
            'maxQuestionLength' => 500,
            'minQuestionLength' => 3,
            'maxAnswerLength' => 150
        )));
        
        OW::getDocument()->addOnloadScript($js);

    }

    public function initForm()
    {
        return new QUESTIONSANSW_AddForm();
    }
}

class QUESTIONSANSW_AddForm extends Form
{
    public function __construct()
    {
        parent::__construct('questionsansw_add');

        $language = OW::getLanguage();

        $this->setAjax();
        $this->setAjaxResetOnSuccess(false);

        $field = new Textarea('question');
        $field->addAttribute('maxlength', 500);
        $field->setRequired();
        $field->setHasInvitation(true);
        $field->setInvitation( $language->text('questionsansw', 'question_add_text_inv') );
        $field->addAttribute( "inv", $language->text('questionsansw', 'question_add_text_inv') );
        
        $this->addElement($field);

        $field = new CheckboxField('allowAddOprions');
        $field->addAttribute('checked');
        $field->setLabel( $language->text('questionsansw', 'question_add_allow_add_opt') );
        $this->addElement($field);

        $field = new QUESTIONSANSW_OptionsField('answers');
        $field->setHasInvitation(true);
        $field->setInvitation( $language->text('questionsansw', 'question_add_option_inv') );
        $this->addElement($field);


        $submit = new Submit('save');
        $submit->setValue($language->text('questionsansw', 'question_add_save'));
        $this->addElement($submit);

        if ( !OW::getRequest()->isAjax() )
        {
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_question_empty');
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_question_min_length');
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_question_max_length');
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_question_two_apt_required');
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_question_dublicate_option');
            OW::getLanguage()->addKeyForJs('questionsansw', 'feedback_option_max_length');

            $this->initJsResponder();
        }

        $this->setAction( OW::getRequest()->buildUrlQueryString(OW::getRouter()->urlFor('QUESTIONSANSW_CTRL_List', 'addQuestion')) );
    }

    public function initJsResponder()
    {
        $js = UTIL_JsGenerator::composeJsString(' owForms["questionsansw_add"].bind( "success", function( r )
        {
            var form = owForms["questionsansw_add"];
            if ( r.reset !== false )
            {
                form.getElement("answers").resetValue();
                form.getElement("question").resetValue();
                QUTILS.addInvitation(form.getElement("question").input);

                OW.trigger("questionsansw.after_question_add", [r]);
            }

            if ( r )
            {
                window.QUESTIONSANSW_ListObject.ajaxSuccess(r);
            }

        });');

        OW::getDocument()->addOnloadScript( $js );
    }
}

/**
 * Form element: TextField.
 *
 * @author Sardar Madumarov <madumarov@gmail.com>
 * @package ow_core
 * @since 1.0
 */
class QUESTIONSANSW_OptionsField extends InvitationFormElement
{
    private $itemIds = array();

    /**
     * @see FormElement::renderInput()
     *
     * @param array $params
     * @return string
     */
    public function renderInput( $params = array() )
    {
        $value = $this->getValue();
        $countValue = empty($value) ? 3 : count($value) + 1;
        $count = $countValue > 3 ? $countValue : 3;
        $content = $this->renderItem(-1, true);

        for ( $i=0; $i < $count; $i++ )
        {
            $content .= $this->renderItem($i);
        }

        return UTIL_HtmlTag::generateTag('div', array_merge($this->attributes, $params), true, $content);
    }

    private function renderItem( $index, $proto = false )
    {
        $value = $this->getValue();

        $inputAttrs = array(
            'type' => 'text',
            'maxlength' => 150,
            'name' => $this->getName() . '[]',
            'class' => 'mt-item-input',
            'value' => empty($value[$index]) ? '' : $value[$index]
        );

        $contAttrs = array(
            'class' => 'mt-item ow_smallmargin'
        );

        if ( $proto )
        {
            $inputAttrs['value'] = '';
            $contAttrs['style'] = 'display: none;';
        }

        if ( $this->getHasInvitation() && empty($inputAttrs['value']) )
        {
            $inputAttrs['value'] = $this->invitation;
            $inputAttrs['class'] .= ' invitation';
        }

        $input = UTIL_HtmlTag::generateTag('input', $inputAttrs);

        return UTIL_HtmlTag::generateTag('div', $contAttrs, true, $input);
    }

    public function getElementJs() {

        $js = UTIL_JsGenerator::newInstance()->newObject('formElement', 'QUESTIONSANSW_AnswersField', array(
            $this->getId(), $this->getName(), ($this->getHasInvitation() ? $this->getInvitation() : false)
        ));

        /** @var $value Validator  */
        foreach ( $this->validators as $value )
        {
             $js .= "formElement.addValidator(" . $value->getJsValidator() . ");";
        }

        return  $js;
    }
}
